@@ -139,13 +139,6 @@
 #include <stdio.h>
 #include <errno.h>
 
-#if defined __OpenBSD__
-#include <sys/types.h>
-#include <unistd.h>
-#include <sys/param.h>
-#include <sys/sysctl.h>
-#endif
-
 #include "goption.h"
 
 #include "gprintf.h"
@@ -363,8 +356,7 @@ void g_option_context_free (GOptionContext *context)
 {
   g_return_if_fail (context != NULL);
 
-  g_list_foreach (context->groups, (GFunc)g_option_group_free, NULL);
-  g_list_free (context->groups);
+  g_list_free_full (context->groups, (GDestroyNotify) g_option_group_free);
 
   if (context->main_group)
     g_option_group_free (context->main_group);
@@ -1672,7 +1664,7 @@ free_pending_nulls (GOptionContext *context,
 static char *
 platform_get_argv0 (void)
 {
-#if defined __linux
+#ifdef __linux
   char *cmdline;
   char *base_arg0;
   gsize len;
@@ -1692,28 +1684,6 @@ platform_get_argv0 (void)
   base_arg0 = g_path_get_basename (cmdline);
   g_free (cmdline);
   return base_arg0;
-#elif defined __OpenBSD__
-  char **cmdline = NULL;
-  char *base_arg0;
-  gsize len = PATH_MAX;
-
-  int mib[] = { CTL_KERN, KERN_PROC_ARGS, getpid(), KERN_PROC_ARGV };
-
-  cmdline = (char **) realloc (cmdline, len);
-
-  if (sysctl (mib, nitems (mib), cmdline, &len, NULL, 0) == -1)
-    {
-      g_free (cmdline);
-      return NULL;
-    }
-
-  /* We could just return cmdline, but I think it's better
-   * to hold on to a smaller malloc block; the arguments
-   * could be large.
-   */
-  base_arg0 = g_path_get_basename (*cmdline);
-  g_free (cmdline);
-  return base_arg0;
 #endif
 
   return NULL;
@@ -2179,16 +2149,14 @@ g_option_group_add_entries (GOptionGroup       *group,
 
       if (c == '-' || (c != 0 && !g_ascii_isprint (c)))
         {
-          g_warning (G_STRLOC ": ignoring invalid short option '%c' (%d) in entry %s:%s",
-              c, c, group->name, group->entries[i].long_name);
-          group->entries[i].short_name = '\0';
+          g_warning (G_STRLOC ": ignoring invalid short option '%c' (%d)", c, c);
+          group->entries[i].short_name = 0;
         }
 
       if (group->entries[i].arg != G_OPTION_ARG_NONE &&
           (group->entries[i].flags & G_OPTION_FLAG_REVERSE) != 0)
         {
-          g_warning (G_STRLOC ": ignoring reverse flag on option of arg-type %d in entry %s:%s",
-              group->entries[i].arg, group->name, group->entries[i].long_name);
+          g_warning (G_STRLOC ": ignoring reverse flag on option of type %d", group->entries[i].arg);
 
           group->entries[i].flags &= ~G_OPTION_FLAG_REVERSE;
         }
@@ -2196,8 +2164,7 @@ g_option_group_add_entries (GOptionGroup       *group,
       if (group->entries[i].arg != G_OPTION_ARG_CALLBACK &&
           (group->entries[i].flags & (G_OPTION_FLAG_NO_ARG|G_OPTION_FLAG_OPTIONAL_ARG|G_OPTION_FLAG_FILENAME)) != 0)
         {
-          g_warning (G_STRLOC ": ignoring no-arg, optional-arg or filename flags (%d) on option of arg-type %d in entry %s:%s",
-              group->entries[i].flags, group->entries[i].arg, group->name, group->entries[i].long_name);
+          g_warning (G_STRLOC ": ignoring no-arg, optional-arg or filename flags (%d) on option of type %d", group->entries[i].flags, group->entries[i].arg);
 
           group->entries[i].flags &= ~(G_OPTION_FLAG_NO_ARG|G_OPTION_FLAG_OPTIONAL_ARG|G_OPTION_FLAG_FILENAME);
         }
